package net.sf.sido;

import java.util.List;
import java.util.Locale;

/**
 * Defines the results for a SiDO validation.
 * @see net.sf.sido.DataObject#validate()
 */
public interface ValidationResults {

    /**
     * Gets the list of errors
     * @return List of errors (can be empty but never null)
     */
	List<ValidationError> getErrors();

    /**
     * Result of the validation
     * @return true if no error was detected, false otherwise
     */
	boolean isValid();

    /**
     * Creates a string representation of this result. The locale used to render
     * the error messages is the default one.
     * @return String
     * @see java.text.MessageFormat
     * @see #getDisplayableResults(java.util.Locale)
     */
    String getDisplayableResults ();

    /**
     * Creates a string representation of this result. The line format is "* {0}\n".
     * @param locale Locale to use to render the error messages
     * @return String
     * @see java.text.MessageFormat
     * @see #getDisplayableResults(java.util.Locale, String)
     */
    String getDisplayableResults (Locale locale);

    /**
     * Creates a string representation of this result. The result format is "{0}".
     * @param locale Locale to use to render the error messages
     * @param lineFormat Format used to render each line ({0} is used for the error message)
     * @return String
     * @see java.text.MessageFormat
     * @see #getDisplayableResults(java.util.Locale, String, String)
     */
    String getDisplayableResults (Locale locale, String lineFormat);

    /**
     * Creates a string representation of this result.
     * @param locale Locale to use to render the error messages
     * @param lineFormat Format used to render each line ({0} is used for the error message)
     * @param resultFormat Format used to render the global result  ({0} is used for the result)
     * @return String
     * @see java.text.MessageFormat
     */
    String getDisplayableResults (Locale locale, String lineFormat, String resultFormat);

    /**
     * Returns the list of localized error messages, using the default locale.
     * @return List of localized messages
     */
    List<String> getDisplayableErrors();

    /**
     * Returns the list of localized error messages
     * @param locale Locale to use to render the error messages
     * @return List of localized messages
     */
    List<String> getDisplayableErrors(Locale locale);

}
