package net.sf.sido.constraint.support;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import net.sf.sido.constraint.ConstraintDefinition;
import net.sf.sido.constraint.ConstraintDefinitions;
import net.sf.sido.constraint.ConstraintFactory;

import com.google.common.base.Predicate;
import com.google.common.base.Predicates;

public class DefaultConstraintFactory implements ConstraintFactory {

	private Map<String, ConstraintParser<?>> parsers = new HashMap<String, ConstraintParser<?>>();

	@SuppressWarnings("rawtypes")
	public DefaultConstraintFactory() {
		parsers.put("maxlength", new StringLengthConstraintParser());
		parsers.put("matches", new MatchesConstraintParser());
		parsers.put("gt", new GTConstraintParser());
		parsers.put("gte", new GTEConstraintParser());
		parsers.put("lt", new LTConstraintParser());
		parsers.put("lte", new LTEConstraintParser());
		parsers.put("enum", new EnumConstraintParser());
		parsers.put("custom", new CustomConstraintParser());
	}

	protected <T> Predicate<T> createConstraint(ConstraintDefinition constraintDefinition) {
		// Gets the type of constraint
		String type = constraintDefinition.getType();
		// Gets the associated handler
		@SuppressWarnings("unchecked")
		ConstraintParser<T> parser = (ConstraintParser<T>) parsers.get(type);
		if (parser == null) {
			throw new DOConstraintTypeNotFoundException(type);
		}
		// Parses the node
		return parser.parse(constraintDefinition);
	}

	@Override
	public <T> Predicate<T> createConstraint(ConstraintDefinitions constraintDefinitions) {
		List<ConstraintDefinition> array = constraintDefinitions.getConstraintDefinitions();
		if (array.isEmpty()) {
			return null;
		} else if (array.size() == 1) {
			return createConstraint(array.get(0));
		} else {
			List<Predicate<T>> constraints = new ArrayList<Predicate<T>>();
			for(ConstraintDefinition constraintDefinition : array) {
				Predicate<T> constraint = createConstraint(constraintDefinition);
				constraints.add(constraint);
			}
			return Predicates.and(constraints);
		}
	}

	public Map<String, ConstraintParser<?>> getParsers() {
		return parsers;
	}

	public void setParsers(Map<String, ConstraintParser<?>> parsers) {
		this.parsers = parsers;
	}

}
