package net.sf.sido.factory.support;

import net.sf.sido.DataObject;
import net.sf.sido.DataSchema;
import net.sf.sido.DataType;
import net.sf.sido.DataTypeBuilder;
import net.sf.sido.array.Array;

public class TypeDefinition {

    private final DataSchema schema;
    private DataObject sType;
    private TypeResolutionStatus status;

	private final DataTypeBuilder builder;
	private DataType type;

    public TypeDefinition(DataSchema schema, DataObject sType, DataTypeBuilder builder) {
        this.schema = schema;
        this.sType = sType;
        this.status = TypeResolutionStatus.PENDING;
        this.builder = builder;
		// Extensibility
        if (sType.getBoolean(SiDOSchema.EXTENSIBLE)) {
            builder.allowExtensions();
        }
		// Abstract type
		if (sType.getBoolean(SiDOSchema.ABSTRACT)) {
			builder.abstractType();
		}
    }

    public String getQualifiedName() {
        return String.format("%s%s%s",
                schema.getName(),
                DataSchema.QUALIFIER_SEPARATOR,
                builder.getName());
    }

    public TypeResolutionStatus getStatus() {
        return status;
    }

	public void start() {
		if (this.status == TypeResolutionStatus.ONGOING || this.status == TypeResolutionStatus.COMPLETE) {
			throw new IllegalStateException(String.format("Status %s for definition %s cannot be started.", this.status, getQualifiedName()));
		} else if (this.status == TypeResolutionStatus.PENDING) {
			this.status = TypeResolutionStatus.ONGOING;
		} else {
			throw new IllegalStateException(String.format("Status %s is unknown.", this.status));
		}
	}

	public void complete() {
		if (this.status == TypeResolutionStatus.COMPLETE || this.status == TypeResolutionStatus.PENDING) {
			throw new IllegalStateException(String.format("Status %s for definition %s cannot be completed.", this.status, getQualifiedName()));
		} else if (this.status == TypeResolutionStatus.ONGOING) {
			this.type = this.builder.getType();
			this.status = TypeResolutionStatus.COMPLETE;
			this.type.close();
		} else {
			throw new IllegalStateException(String.format("Status %s is unknown.", this.status));
		}
	}

	public DataType getType() {
        if (type == null) {
            throw new IllegalStateException(String.format("Type %s is not defined yet.", getQualifiedName()));
        }
		return this.type;
	}

    public DataTypeBuilder getBuilder() {
        return builder;
    }

    /**
     * Gets the qualified name of the parent type, or null if not defined.
     */
    public String getParentTypeName() {
        String typeName = sType.getString(SiDOSchema.PARENT);
        if (typeName != null) {
            return getQualifiedName(typeName);
        } else {
            return null;
        }
    }

    public String getQualifiedName(String typeName) {
        if (typeName.contains(DataSchema.QUALIFIER_SEPARATOR)) {
            return typeName;
        } else {
            return String.format("%s%s%s",
                    schema.getName(),
                    DataSchema.QUALIFIER_SEPARATOR,
                    typeName);
        }
    }

    public void setParentType(DataType parentType) {
        builder.setParentType(parentType);
    }

    public Array<DataObject> getProperties() {
        return sType.getDOArray(SiDOSchema.PROPERTIES);
    }
}
