package net.sf.sido.io.xml;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import net.sf.sido.DataSchema;
import net.sf.sido.DataType;
import net.sf.sido.Property;

import org.apache.commons.lang.StringUtils;
import org.dom4j.Namespace;
import org.dom4j.QName;

public class DefaultXMLDataObjectNamespace implements XMLDataObjectNamespace {

	private final Map<String, Namespace> namespaces = new ConcurrentHashMap<String, Namespace>();

	private Namespace createNamespace(String name) {
		return new Namespace(StringUtils.lowerCase(name), XMLDOSchema.NS_SDO_URI + "/" + name);
	}

	@Override
	public synchronized Namespace getNamespace(DataSchema schema) {
		String name = schema.getName();
		Namespace ns = namespaces.get(name);
		if (ns != null) {
			return ns;
		} else {
			ns = createNamespace(name);
			namespaces.put(name, ns);
			return ns;
		}
	}

	@Override
	public QName getQName(Property<?> property) {
		DataType holder = property.getHolderType();
		DataSchema schema = holder.getSchema();
		Namespace ns = getNamespace(schema);
		return new QName(property.getName(), ns);
	}

}
